import os
import sys
import datetime
import common

#this means ONLY CO2
CO2 = (len(sys.argv) >= 5 and sys.argv[4] == "co2")
WITH_DAPP = (len(sys.argv) >= 6)
if WITH_DAPP:
    DAPP_PARAM = float(sys.argv[5])
    DISTR_COST = 0.12
    NPP_SCENARIO_ID = int(sys.argv[6])
    COPY = int(sys.argv[7])
    ALL_COPIES = 3
PRICE_PER_KWH = 90 if len(sys.argv) < 2 else float(sys.argv[1])
NUM_YEARS = 10.0 if len(sys.argv) < 4 else float(sys.argv[3])
PRICE_PER_KW = 200 if len(sys.argv) < 3 else float(sys.argv[2])
TAXES = 2.875
HOURS_IN_DAY = 24
DAYS_IN_YEAR = 365

dir_out = "without_pev_ctrl.milps"

def get_min_max_ids(dir_in):
    ids = []
    for dir_h in os.listdir(dir_in):
        if dir_h.find("home") == 0:
            ids += [int(dir_h.split("_")[1])]
    return (min(ids), max(ids))

def retrieve_datetime(s):
    if s.find(" ") != -1:
        s_ar = s.split(" ")
    else:
        s_ar = s.split("T")
    d_ar = s_ar[0].split("-")
    t_ar = s_ar[1].split(":")
    return datetime.datetime(int(d_ar[0]), int(d_ar[1]), int(d_ar[2]), int(t_ar[0]))

def compose_datetime(record):
    return datetime.datetime(record["starting_utc"].year, record["starting_utc"].month, record["starting_utc"].day, record["starting"].hour)

def create_dict(prices, pos = -1):
    res = {}
    for line in prices[1:]:
        line_ar = line.split(",")
        res[retrieve_datetime(line_ar[0])] = float(line_ar[pos] if line_ar[pos] != "-\n" and line_ar[pos] != "-" else 0)
    return res

def price_item(prices_d, lines, i, with_taxes = True):
    try:
        return max(float(prices_d[retrieve_datetime(lines[i].split(",")[0])]), 0)*(TAXES if with_taxes else 1)
    except Exception as ex:
        pass

def retrieve_price_policy(home_id, copy, npp_scenario, from_h, to_h, copies):
    LOCAL_DB = "local_dapp_third_deliv"
    LOCAL_DB_USER = "local_dapp_third_deliv_user"
    LOCAL_DB_USER_PWD = "local_dapp_third_deliv_user_password"
    (conn, db) = common.db_connect("dbname=%s host=localhost user=%s password=%s" %(LOCAL_DB, LOCAL_DB_USER, LOCAL_DB_USER_PWD))
    res = common.exec_and_retrieve(db, "select dodu.starting at time zone 'utc' as starting_utc, power_usage, h.max_pow, h.min_pow, iss.description from dapp_output_dapp_user dodu inner join dapp_execution dex on dex.id = dodu.exec_id inner join home h on h.id = dodu.home_id inner join input_subs_scenario iss on iss.id = dex.npp_scenario_id where h.address = 'Copy %d of home %d' and dex.npp_scenario_id = %d order by starting_utc" %(copy, home_id, npp_scenario))
    res2 = common.exec_and_retrieve(db, "select dodu.starting at time zone 'utc' as starting_utc, sum(power_usage)*%d as aggregate from dapp_output_dapp_user dodu inner join dapp_execution dex on dex.id = dodu.exec_id where dex.npp_scenario_id = %d and home_id >= %d and home_id <= %d group by starting_utc order by starting_utc" %(copies, npp_scenario, from_h - from_h + 1, to_h - from_h + 1))
    res3 = common.exec_and_retrieve(db, "select sum(max_pow)*%d as all_contracts from home where id >= %d and id <= %d" %(copies, from_h - from_h + 1, to_h - from_h + 1))
    db.close()
    conn.close()
    limits = {"lower": float(res[0]["description"].split("[")[1].split(",")[0]), "upper": float(res[0]["description"].split(", ")[1].split("]")[0])}
    pp = []
    for i in range(len(res)):
        assert(res[i]["starting_utc"] == res2[i]["starting_utc"])
        pp += [{"starting_utc": res[i]["starting_utc"], "min_p": res[i]["power_usage"] - max((res2[i]["aggregate"] - limits["lower"])/((to_h + 1 - from_h)*copies), 0), "max_p": res[i]["power_usage"] + max((-res2[i]["aggregate"] + limits["upper"])*res[i]["max_pow"]/res3[0]["all_contracts"], 0)}]
    # res = common.exec_and_retrieve(db, "select starting_val at time zone 'utc' as starting_utc, dout.starting, min_p, max_p from dapp_output dout inner join dapp_execution dex on dex.id = dout.exec_id inner join home h on h.id = dout.home_id where h.address = 'Copy %d of home %d' and dex.npp_scenario_id = %d order by starting_utc, starting" %(copy, home_id, npp_scenario))
    return pp

def one_year_factor(num_hours):
    return num_hours/(float(HOURS_IN_DAY)*DAYS_IN_YEAR)

f = open("input_ebr/market.csv", "r")
prices = f.readlines()
f.close()
prices_d = create_dict(prices, -1)
co2_d = create_dict(prices, -3)
co2_emissions_d = create_dict(prices, -4)
(from_h, to_h) = get_min_max_ids("input_ebr/homes")
batteries = {}
batteries_costs = []
try:
    f = open("fix_batteries.csv", "r")
    lines = f.readlines()
    f.close()
    for line in lines[1:]:
        if int(line.split(",")[0]) == from_h:
            batteries_costs += [(float(line.split(",")[1]), float(line.split(",")[2]), float(line.split(",")[3]))]
        home_fix_batteries = int(line.split(",")[0])
        if home_fix_batteries not in batteries:
            batteries[home_fix_batteries] = [(float(line.split(",")[8]), float(line.split(",")[9]))]
        else:
            batteries[home_fix_batteries] += [(float(line.split(",")[8]), float(line.split(",")[9]))]
    # batteries = [float(line.split(",")[6]) for line in lines]
except Exception as ex:
    batteries_costs = [(PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS)]
for h in range(from_h, to_h + 1):
    for batt_i in range(len(batteries_costs)): 
#    for (PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS) in batteries_costs:
        (PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS) = batteries_costs[batt_i]
        print h, PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS
        try:
            f = open("input_ebr/homes/home_%d/profiles.csv" %h, "r")
            lines = f.readlines()
            f.close()
        except Exception as ex:
            continue
        f = open("input_ebr/homes/home_%d/energy.csv" %h, "r")
        energy = f.readlines()[1]
        f.close()
        if WITH_DAPP:
            pp = retrieve_price_policy(h, COPY, NPP_SCENARIO_ID, from_h, to_h, ALL_COPIES)
            basefilename = "home_%d.kwh.%lf.kw.%lf.y.%lf.copy.%d.npp_scen.%d.p.%lf." %(h, PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS, COPY, NPP_SCENARIO_ID, DAPP_PARAM)
        else:
            basefilename = "home_%d.kwh.%lf.kw.%lf.y.%lf.%s" %(h, PRICE_PER_KWH, PRICE_PER_KW, NUM_YEARS, "co2." if CO2 else "")
        try:
            os.makedirs(dir_out)
        except Exception as ex:
            pass
        output = open("%s/%slp" %(dir_out, basefilename), "w")
        if not(CO2):
            output.write("Minimize\n  obj: %+lf Q %+lf PR " %(PRICE_PER_KWH*one_year_factor(len(lines) - 1)/NUM_YEARS, PRICE_PER_KW*one_year_factor(len(lines) - 1)/NUM_YEARS))
        else:
            output.write("Minimize\n  obj: ")
        if (WITH_DAPP):
            for index_pp in range(1, len(lines)):
                if retrieve_datetime(lines[index_pp].split(",")[0]) < pp[0]["starting_utc"]:
                    continue
                else:
                    break
        else:
            index_pp = 1
        for i in range(index_pp, len(lines)):
            if (WITH_DAPP):
                output.write("%+lg y_%d %+lg deltaplus_%d %+lg deltaminus_%d " %((price_item(co2_d, lines, i) + price_item(prices_d, lines, i)), i, DAPP_PARAM*DISTR_COST, i, DAPP_PARAM*DISTR_COST, i))
            elif not(CO2):
                output.write("%+lg y_%d " %(price_item(prices_d, lines, i) + price_item(co2_d, lines, i), i))
            else:
                output.write("%+lg y_%d " %(price_item(co2_d, lines, i), i))
        output.write("\nSubject To:\n  b_1_%d - 0.5 Q = 0\n" %index_pp)
        if batteries != {}:
            output.write("  Q = %lf\n  PR = %lf\n" %(batteries[h][batt_i][0], batteries[h][batt_i][1]))
        cost_without_ebr_co2 = 0
        cost_with_ebr_co2_str = "-cost_with_ebr_co2"
        emissions_with_ebr_str = "-emissions_with_ebr"
        emissions_without_ebr = 0
        cost_without_ebr_en = 0
        if not(CO2):
            cost_with_ebr_en_str = "-cost_with_ebr_en"
        if WITH_DAPP:
            all_deltaplus_str = "-all_deltaplus"
            all_deltaminus_str = "-all_deltaminus"
            all_deltaplus_without_ebr = 0
            all_deltaminus_without_ebr = 0
        annual_consumption_for_cost  = 0
        annual_consumption = 0
        for i in range(index_pp, len(lines)):
            line_ar = lines[i].split(",")
            demand = float(line_ar[2]) - float(line_ar[3]) + float(line_ar[5] if line_ar[5] != "" else "0")
            demand_for_cost = max(demand, 0)
            if WITH_DAPP:
                output.write("  e_%d + deltaminus_%d >= %lf\n" %(i, i, pp[i - index_pp]["min_p"]))
                output.write("  e_%d - deltaplus_%d <= %lf\n" %(i, i, pp[i - index_pp]["max_p"]))
                demand_without_ebr = demand
                all_deltaplus_without_ebr += max(demand_without_ebr - pp[i - index_pp]["max_p"], 0)
                all_deltaminus_without_ebr += max(-demand_without_ebr + pp[i - index_pp]["min_p"], 0)
                all_deltaplus_str += " +deltaplus_%d" %i
                all_deltaminus_str += " +deltaminus_%d" %i
            cost_without_ebr_co2 += demand_for_cost*(price_item(co2_d, lines, i))
            cost_with_ebr_co2_str += " %+lg y_%d" %(price_item(co2_d, lines, i), i)
            emissions_without_ebr += demand_for_cost*(price_item(co2_emissions_d, lines, i, False))
            emissions_with_ebr_str += " %+lg y_%d" %(price_item(co2_emissions_d, lines, i, False), i)
            if not(CO2):
                cost_without_ebr_en += demand_for_cost*(price_item(prices_d, lines, i))
                cost_with_ebr_en_str += " %+lg y_%d" %(price_item(prices_d, lines, i), i)
            annual_consumption_for_cost += demand_for_cost
            annual_consumption += demand
            output.write("  e_%d - a_1_cd_%d = %lf\n" %(i, i, demand))
            output.write("  y_%d - e_%d >= 0\n" %(i, i))
            output.write("  y_%d >= 0\n" %i)
            output.write("  b_1_%d - b_1_%d - a_1_cd_%d = 0\n  a_1_cd_%d - PR <= 0\n  a_1_cd_%d + PR >= 0\n  b_1_%d - Q <= 0\n" %(i + 1, i, i, i, i, i + 1))
            if demand >= 0:
                output.write("  a_1_cd_%d >= %lf\n" %(i, -demand))
            else:
                output.write("  a_1_cd_%d >= 0\n" %i)
        output.write("  cost_without_ebr = %lf\n  annual_consumption_for_cost = %lf\n  annual_consumption = %lf\n" %(cost_without_ebr_co2 + cost_without_ebr_en, annual_consumption_for_cost, annual_consumption))
        output.write("  cost_without_ebr_co2 = %lf\n" %(cost_without_ebr_co2))
        output.write("  %s = 0\n" %cost_with_ebr_co2_str)
        if not(CO2):
            output.write("  cost_without_ebr_en = %lf\n" %(cost_without_ebr_en))
            output.write("  %s = 0\n" %cost_with_ebr_en_str)
        if WITH_DAPP:
            output.write("  all_deltaplus_without_ebr = %lf\n  all_deltaminus_without_ebr = %lf\n" %(all_deltaplus_without_ebr, all_deltaminus_without_ebr))
            output.write("  %s = 0\n" %all_deltaplus_str)
            output.write("  %s = 0\n" %all_deltaminus_str)
        output.write("  %s = 0\n  emissions_without_ebr = %lf\n" %(emissions_with_ebr_str, emissions_without_ebr))
        output.write("Bounds\n  0 <= b_1_%d <= 10000\n  0 <= Q <= 10000\n  0 <= PR <= 10000\n" %index_pp)
        for i in range(index_pp, len(lines)):
            output.write("  %lf <= e_%d <= %lf\n  0 <= y_%d <= %lf\n  0 <= b_1_%d <= 10000\n  -10000 <= a_1_cd_%d <= 10000\n" %(float(energy.split(",")[1]), i, float(energy.split(",")[0]), i, float(energy.split(",")[0]), i + 1, i))
            if WITH_DAPP:
                output.write("  0 <= deltaplus_%d <= inf\n  0 <= deltaminus_%d <= inf\n" %(i, i))
        output.write("  cost_without_ebr_co2 = %lf\n" %(cost_without_ebr_co2))
        output.write("  cost_with_ebr_co2 free\n")
        if not(CO2):
            output.write("  cost_without_ebr_en = %lf\n" %(cost_without_ebr_en))
            output.write("  cost_with_ebr_en free\n")
        if WITH_DAPP:
            output.write("  all_deltaplus_without_ebr = %lf\n  all_deltaminus_without_ebr = %lf\n" %(all_deltaplus_without_ebr, all_deltaminus_without_ebr))
            output.write("  all_deltaplus free\n")
            output.write("  all_deltaminus free\n")
        output.write("  cost_without_ebr = %lf\n  annual_consumption_for_cost = %lf\n  annual_consumption = %lf\n" %(cost_without_ebr_co2 + cost_without_ebr_en, annual_consumption_for_cost, annual_consumption))
        output.write("  emissions_with_ebr >= 0\n  emissions_without_ebr = %lf\n" %(emissions_without_ebr))
        output.write("End\n")
        output.close()
        output = open("%s/%scplex.cmd" %(dir_out, basefilename), "w")
        output.write("read %slp\noptimize\nwrite %scplex.sol\nquit\n" %(basefilename, basefilename))
        output.close()
