#define  _BSD_SOURCE
#include <EBR.h>
#include <HECH.h>
#include <Properties.h>
#include <Debug.h>

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>



#define DEBUG "EBR"



#ifndef FAST_EXPS
HECH * EBR_houseConnect(Properties *conf)
#else
HECH * EBR_houseConnect(Properties *conf, CommandLineInterface * cliPtr, House **myHouse)
#endif
{
	HECH * myHECH = NULL;
	const char * houseIPAddr;
	int measPort;
	int cmdsPort;
	int controlSignalStop;

#ifndef FAST_EXPS
	if (Properties_getString(conf, "HECH_IP", &houseIPAddr) < 0) {
		fprintf(stderr, "\nAdd in configuration file a line\n\nstring HECH_IP = aString\n");
		exit(1);
	}

	if (Properties_getInt(conf, "HECH_MEASUREMENTS_PORT", &measPort) < 0) {
		fprintf(stderr, "\nAdd in configuration file a line\n\nint HECH_MEASUREMENTS_PORT = anInteger\n");
		exit(1);
	}

	if (Properties_getInt(conf, "HECH_COMMANDS_PORT", &cmdsPort) < 0) {
		fprintf(stderr, "\nAdd in configuration file a line\n\nint HECH_COMMANDS_PORT = anInteger\n");
		exit(1);
	}
#endif

	if (Properties_getInt(conf, "controlSignalStop", &controlSignalStop) < 0) {
		fprintf(stderr, "\nAdd in configuration file a line\n\nint controlSignalStop = anInteger\n");
		exit(1);
	}

#ifndef FAST_EXPS
	myHECH = HECH_new(houseIPAddr, measPort, cmdsPort, controlSignalStop);
#else
	char profilesFile[PATH_MAX];
	char loggingFile[PATH_MAX];
	char batteryFile[PATH_MAX];
	char phevFile[PATH_MAX];
	int isSubstation, tsl;
	realpath(CommandLineInterface_getStringOption(cliPtr, "profilesFile"), profilesFile);
	realpath(CommandLineInterface_getStringOption(cliPtr, "hLogFile"), loggingFile);
	realpath(CommandLineInterface_getStringOption(cliPtr, "batteryFile"), batteryFile);
	realpath(CommandLineInterface_getStringOption(cliPtr, "phevFile"), phevFile);
	isSubstation = CommandLineInterface_getFlag(cliPtr, "isSubstation");
	tsl = CommandLineInterface_getIntOption(cliPtr, "timeslotLength");
	/* infESSCapacity = CommandLineInterface_getFlag(cliPtr, "infESSCapacity"); */
	myHECH = HECH_new(controlSignalStop, myHouse, conf, profilesFile, loggingFile, batteryFile, phevFile, isSubstation, tsl);
	/* if (conf != NULL) Properties_free(&conf); */
#endif
	return myHECH;
}




#ifndef FAST_EXPS
void EBR_houseDisconnect(HECH **myHECH)
{
  HECH_free(myHECH);
}
#else
void EBR_houseDisconnect(HECH **myHECH, House **myHouse)
{
  HECH_free(myHECH, myHouse);
}
#endif





