#define _GNU_SOURCE

#include <EBR_milp.h>
#include <EBR.h>


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include <stdarg.h>


#define DEBUG "EBR MILP"


static int _EBR_file_exists(char *filename);


void EBR_milp_monomial(char ** names, double * vals, int index, double coefficient, char *fmt, ...)
{
	Debug_assert(DEBUG, names != NULL, "names == NULL\n");
	Debug_assert(DEBUG, vals != NULL, "vals == NULL\n");
	char var_name[1024];
	va_list args;
	va_start(args, fmt);
	vsprintf(var_name, fmt, args);
	names[index] = strdup(var_name);
	vals[index] = coefficient;
	va_end(args);
}


void EBR_milp_monomial_dispose(char ** names, int howmany)
{
	int i;
	for (i=0; i<howmany; i++) {
		free(names[i]);
	}
}


void EBR_milp_solver(MILPSolver_t res, char * solver)
{
	Debug_assert(DEBUG, solver != NULL, "solver == NULL\n");
	switch (res) {
		case MILPSOLVER_NONE:
			sprintf(solver, "NONE");
			break;
		case MILPSOLVER_CPLEX:
			sprintf(solver, "CPLEX");
			break;
		case MILPSOLVER_GLPSOL:
			sprintf(solver, "GLPK");
			break;
		default:
			break;
	}
}


int EBR_milp_get_solution(MILPSolver_t res, char * dir, char * filename, char * filename_log, double * commands, EBR_solution *solution)
{
	int out = 0;

	if (res == MILPSOLVER_NONE) {
		out = -1;
	}
	else if (res == MILPSOLVER_CPLEX) {
		static char buffer[MAX_BUF_LEN];
		FILE *stream;
		double a1c1 = 0;
		double a1d1 = 0;
		double a2c2 = 0;
		double a2d2 = 0;
		int res;

		assert(snprintf(buffer, MAX_BUF_LEN, "%s/%s", dir, filename_log) < MAX_BUF_LEN);
		if (!(stream = fopen(buffer, "r"))) {
			fprintf(stderr, "Impossible to open file %s: ", filename_log);
			perror("");
			exit(EXIT_FAILURE);
		}
		res = 1;
		while (res && fgets(buffer, MAX_BUF_LEN, stream)) {
			if (strstr(buffer, "No solution exists") != (char *)0 || strstr(buffer, "nfeasible") != (char *)0)
				res = 0;
		}
		fclose(stream);
		if (!res) {
			commands[CMDS_BATTERY] = 0;
			commands[CMDS_PHEV] = 0;
			bzero(solution, sizeof(EBR_solution));
			return -1;
		}
		Debug_out(DEBUG, "Parsing file %s/%s\n", dir, filename);
		assert(snprintf(buffer, MAX_BUF_LEN, "%s/%s", dir, filename) < MAX_BUF_LEN);
		if (!(stream = fopen(buffer, "r"))) {
			commands[CMDS_BATTERY] = 0;
			commands[CMDS_PHEV] = 0;
			bzero(solution, sizeof(EBR_solution));
			return -2;
		}
		while (fgets(buffer, MAX_BUF_LEN, stream)) {
			char *tmp, *var_name;
			double *set_this;
			if (strstr(buffer, "  <variable name=\"b_1_2\"") == (char *)0 && strstr(buffer, "  <variable name=\"b_2_2\"") == (char *)0 && strstr(buffer, "  <variable name=\"a_1_c_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"y_1_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"y_2_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"a_1_d_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"a_2_c_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"a_2_d_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"deltaplus_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"deltaminus_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"e_1\"") == (char *)0 && strstr(buffer, "  <variable name=\"tilde_e_1\"") == (char *)0)
				continue;
			if (strstr(buffer, "  <variable name=\"a_1_c_1\""))
				set_this = &a1c1;
			else if (strstr(buffer, "  <variable name=\"a_1_d_1\""))
				set_this = &a1d1;
			else if (strstr(buffer, "  <variable name=\"a_2_c_1\""))
				set_this = &a2c2;
			else if (strstr(buffer, "  <variable name=\"a_2_d_1\""))
				set_this = &a2d2;
			else if (strstr(buffer, "  <variable name=\"b_1_2\""))
				set_this = &solution->b_1;
			else if (strstr(buffer, "  <variable name=\"b_2_2\""))
				set_this = &solution->b_2;
			else if (strstr(buffer, "  <variable name=\"y_1_1\""))
				set_this = &solution->y_1;
			else if (strstr(buffer, "  <variable name=\"y_2_1\""))
				set_this = &solution->y_2;
			else if (strstr(buffer, "  <variable name=\"deltaplus_1\""))
				set_this = &solution->dp;
			else if (strstr(buffer, "  <variable name=\"deltaminus_1\""))
				set_this = &solution->dm;
			else if (strstr(buffer, "  <variable name=\"e_1\""))
				set_this = &solution->e;
			else if (strstr(buffer, "  <variable name=\"tilde_e_1\""))
				set_this = &solution->tilde_e;
			var_name = strtok(buffer, "\"");
			do {
				var_name = strtok((char *)0, "\"");
			} while (strstr(var_name, "value") == (char *)0);
			var_name = strtok((char *)0, "\"");
			*set_this = atof(var_name);
		}
		fclose(stream);
		commands[CMDS_BATTERY] = a1d1 > 0?  -a1d1 : a1c1;
		commands[CMDS_PHEV] = a2d2 > 0?  -a2d2 : a2c2;
		Debug_out(DEBUG, "commands[CMDS_BATTERY] = %lg\n", commands[CMDS_BATTERY]);
		Debug_out(DEBUG, "commands[CMDS_PHEV] = %lg\n", commands[CMDS_PHEV]);
		Debug_out(DEBUG, "energy from grid = %lg\n", solution->e);
		out = 0;
	}
	else if (res == MILPSOLVER_GLPSOL) {
		static char buffer[MAX_BUF_LEN];
		FILE *stream;
		double a1c1 = 0;
		double a1d1 = 0;
		double a2c2 = 0;
		double a2d2 = 0;

		Debug_out(DEBUG, "Parsing file %s/%s\n", dir, filename);
		assert(snprintf(buffer, MAX_BUF_LEN, "%s/%s", dir, filename) < MAX_BUF_LEN);
		if (!(stream = fopen(buffer, "r"))) {
			commands[CMDS_BATTERY] = 0;
			commands[CMDS_PHEV] = 0;
			bzero(solution, sizeof(EBR_solution));
			return -2;
		}
		while (fgets(buffer, MAX_BUF_LEN, stream)) {
			if (strstr(buffer, "SOLUTION IS INFEASIBLE") || strstr(buffer, "UNDEFINED")) {
				fclose(stream);
				commands[CMDS_BATTERY] = 0;
				commands[CMDS_PHEV] = 0;
				bzero(solution, sizeof(EBR_solution));
				return -1;
			}
			char *tmp = strtok(buffer, " \t\n");
			if (!tmp)
				continue;
			tmp = strtok(NULL, " \t\n");
			if (tmp) {
				if (strcmp(tmp, "a_1_c_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					a1c1 = atof(tmp);
				}
				else if (strcmp(tmp, "a_1_d_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					a1d1 = atof(tmp);
				}
				else if (strcmp(tmp, "a_2_c_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					a2c2 = atof(tmp);
				}
				else if (strcmp(tmp, "a_2_d_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					a2d2 = atof(tmp);
				}
				else if (strcmp(tmp, "y_1_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->y_1 = atof(tmp);
				}
				else if (strcmp(tmp, "y_2_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->y_2 = atof(tmp);
				}
				else if (strcmp(tmp, "b_1_2") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->b_1 = atof(tmp);
				}
				else if (strcmp(tmp, "b_2_2") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->b_2 = atof(tmp);
				}
				else if (strcmp(tmp, "e_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->e = atof(tmp);
				}
				else if (strcmp(tmp, "tilde_e_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->e = atof(tmp);
				}
				else if (strcmp(tmp, "deltaplus_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->dp = atof(tmp);
				}
				else if (strcmp(tmp, "deltaminus_1") == 0) {
					tmp = strtok(NULL, " \t\n");
					solution->dm = atof(tmp);
				}
			}
		}
		fclose(stream);
		commands[CMDS_BATTERY] = a1d1 > 0?  -a1d1 : a1c1;
		commands[CMDS_PHEV] = a2d2 > 0?  -a2d2 : a2c2;
		Debug_out(DEBUG, "commands[CMDS_BATTERY] = %lg\n", commands[CMDS_BATTERY]);
		Debug_out(DEBUG, "commands[CMDS_PHEV] = %lg\n", commands[CMDS_PHEV]);
		Debug_out(DEBUG, "energy from grid = %lg\n", solution->e);
		out = 0;
	}
	else {
		fprintf(stderr, "Solution format not supported\n");
		out = -1;
	}

	if (out < 0) {
		commands[CMDS_BATTERY] = 0;
		commands[CMDS_PHEV] = 0;
		bzero(solution, sizeof(EBR_solution));
	}

	return out;
}



void EBR_create_var_for_milp(FILE *stream_conv, FILE *stream_vars_list, FILE *stream_vars_bnds, const char *var_name, double lb, double ub)
{
	fprintf(stream_conv, "%s := %lf .. %lf\n", var_name, lb, ub);
	if (stream_vars_list)
		fprintf(stream_vars_list, "  %s\n", var_name);
	fprintf(stream_vars_bnds, "  %lf <= %s <= %lf\n", lb, var_name, ub);
}


void EBR_milp_row_from_monomials(FILE *stream_conv, int bnds_type, double lb, double ub, char *guard, int negated, unsigned num_mons, char **varnames_for_row, double *vals_for_row)
{
	unsigned i;

	if (guard)
		fprintf(stream_conv, "  %s%s -> ", negated? "!" : "", guard);
	for (i = 0; i < num_mons; i++)
		fprintf(stream_conv, "%+lf %s ", vals_for_row[i], varnames_for_row[i]);
	switch (bnds_type) {
		case BOUNDS_TYPE_UB:
			fprintf(stream_conv, "<= %lf\n", ub);
			break;
		case BOUNDS_TYPE_LB:
			fprintf(stream_conv, ">= %lf\n", lb);
			break;
		case BOUNDS_TYPE_FIXED:
			fprintf(stream_conv, "= %lf\n", lb);
			break;
	}

	EBR_milp_monomial_dispose(varnames_for_row, num_mons);
}


int EBR_milp_is_exec(const char *prog, char *cmd, unsigned cmd_length, char *dir)
{
	FILE *stream;
	int ret;

	setbuf(stdout, NULL);
	setbuf(stderr, NULL);
	assert(snprintf(cmd, cmd_length, "which %s > %s/res", prog, dir) < cmd_length);
	system(cmd);
	sprintf(cmd, "%s/res", dir);
	stream = fopen(cmd, "r");
	if (!stream)
		return 0;
	ret = (fscanf(stream, "%c", cmd) != EOF);
	fclose(stream);
	return ret;
}


/**
 *	Stores in @param cmd the name of output file
 *	Returns 1 if solved with CPLEX
 *	Returns 2 if solved with GLPSOL
 *	Returns 0 otherwise
 */
int EBR_milp_solve_file(char *namefile_noext, char *dir, char *cmd, char * filename_log, unsigned cmd_length)
{
	/* tries with cplex first, if not available with glpsol */
	char *executable;

	if (EBR_milp_is_exec("cplex", cmd, cmd_length, dir)) {
		assert(snprintf(cmd, cmd_length, "cd %s\nrm -f %s.cplex.sol\ncat > %s.cplex.cmd << xelpc\nread %s.lp\noptimize\nwrite %s.cplex.sol\nquit\nxelpc\ndate \"+%%F %%H:%%M:%%S%%:z\" > %s.cplex.script.log\n/usr/bin/time -f \"%%e seconds\" -a -o %s.cplex.script.log cplex -f %s.cplex.cmd > %s.cplex.log 2>&1\ntest -f %s.cplex.sol && cat %s.cplex.sol | awk '{apos = sprintf(\"%%c\", 39); gsub(\"&apos;\", apos); print;}' > %s.cplex.apos.sol\n",
						dir, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext) < cmd_length);
		system(cmd);
		/* sotres name of output file */
		sprintf(cmd, "%s.cplex.sol", namefile_noext);
		sprintf(filename_log, "%s.cplex.log", namefile_noext);
		return MILPSOLVER_CPLEX;
	}
	else if (EBR_milp_is_exec("glpsol", cmd, cmd_length, dir)) {
//		assert(snprintf(cmd, cmd_length, "cd %s\ndate \"+%%F %%H:%%M:%%S%%:z\" > %s.glpsol.script.log\n/usr/bin/time -f \"%%e seconds\" -a -o %s.glpsol.script.log glpsol --lp -o %s.glpsol.sol %s.lp > %s.glpsol.log 2>&1\n",
//		    dir, namefile_noext, namefile_noext, namefile_noext, namefile_noext, namefile_noext) < cmd_length);
		assert(snprintf(cmd, cmd_length, "cd %s\ndate \"+%%F %%H:%%M:%%S%%:z\" > %s.glpsol.script.log\nexport LD_LIBRARY_PATH=/usr/local/lib/\nglpsol --cpxlp -o %s.glpsol.sol %s.lp > %s.glpsol.log 2>&1\n",
		    dir, namefile_noext, namefile_noext, namefile_noext, namefile_noext) < cmd_length);
		system(cmd);
		/* stores name of output file */
		sprintf(cmd, "%s.glpsol.sol", namefile_noext);
		sprintf(filename_log, "%s.glpsol.log", namefile_noext);
		return MILPSOLVER_GLPSOL;
	}
	return MILPSOLVER_NONE;
}


int EBR_milp_invoke_convexer(char *filename_noext, char *dir, char *cmd, unsigned cmd_length)
{
	char *convexer = (char *) calloc(strlen(CONVEXER_PATH) + 20, sizeof(char));
	sprintf(convexer, "%s/convexer.py", CONVEXER_PATH);
	if (!_EBR_file_exists(convexer)) {
		free(convexer);
		printf("%s/convexer.py doesn't exist\n", CONVEXER_PATH);
		return -2;
	}
	free(convexer);
	if (EBR_milp_is_exec("python", cmd, cmd_length, dir)) { /* easier than using Python C functions */
		assert(snprintf(cmd, cmd_length, "python %s/convexer.py -V 2 -f %s/%s.clp > %s/%s.clp.res 2> /dev/null\n", CONVEXER_PATH, dir, filename_noext, dir, filename_noext) < cmd_length);
		system(cmd);
		return 1;
	}
	printf("python doesn't exist\n");
	return -1;
}






static int _EBR_file_exists(char *filename)
{
	FILE *stream;
	if (!(stream = fopen(filename, "r")))
		return 0;
	fclose(stream);
	return 1;
}

