#ifndef __EBR_MILP_H__
#define __EBR_MILP_H__

#include <stdio.h>
#include <stdarg.h>
#include "EBR.h"
#define MILP_DIR "milps"

#define WAIT_SOLUTION 10




typedef enum boundstype_t {
	BOUNDS_TYPE_LB = 0,
	BOUNDS_TYPE_UB,
	BOUNDS_TYPE_LB_UB,
	BOUNDS_TYPE_FREE,
	BOUNDS_TYPE_FIXED,
	NUM_BOUNDS_TYPE
} BoundsType_t;

typedef enum milpsolver_t {
	MILPSOLVER_NONE = 0,
	MILPSOLVER_CPLEX,
	MILPSOLVER_GLPSOL
} MILPSolver_t;


void EBR_milp_monomial(char ** names, double * vals, int index, double coefficient, char *fmt, ...);

void EBR_milp_monomial_dispose(char ** names, int howmany);

void EBR_milp_solver(MILPSolver_t res, char * solver);


int EBR_milp_get_solution(MILPSolver_t res, char * dir, char * filename, char * filename_log, double * commands, EBR_solution *);


/**
 *	EBR_create_var_for_milp(stream_conv, stream_vars_list, stream_vars_bnds, var_name, 0, U_Delta_obj);
 *	- stream_vars_list solo per variabili boolean, (FILE *)0 per variabili reali
 *	- ultimi due parametri sono lower e upper bound
 */

void EBR_create_var_for_milp(FILE *stream_conv,
							 FILE *stream_vars_list,
							 FILE *stream_vars_bnds,
							 const char *var_name,
							 double lb, double ub);

/**
 *	For a constraint of length n
 *	@param num_mons is the number of monomials in the constraint
 *	@param varnames_for_row[i] is the variable name of i-th constraint monomial
 *	@param vals_for_row[i] is its coefficient
 *
 *	Example:
 *
 *	- Guarded constraint:	y_i -> x_i <= 0
 *		char * varnames_for_row[MAX_MONOMIALS_NUMBER];
 *		double vals_for_row[MAX_MONOMIALS_NUMBER];
 *		char varname[MAX_NAME_LENGTH];
 *		sprintf(varnames_for_row[0], "x_%d", i);
 *		vals_for_row[0] = 1.0;
 *		sprintf(var_name, "y_%d", i);
 *		EBR_milp_row_from_monomials(stream_conv, BOUNDS_TYPE_UB, 0, 0, var_name, 0, 1, varnames_for_row, vals_for_row);
 *
 * - Constraint:	x_j >= 10
 *		char * varnames_for_row[MAX_MONOMIALS_NUMBER];
 *		double vals_for_row[MAX_MONOMIALS_NUMBER];
 *		sprintf(varnames_for_row[0], "x_%d", d);
 *		vals_for_row[0] = 1.0;
 *		EBR_milp_row_from_monomials(stream_conv, BOUNDS_TYPE_LB, 10, 0, NULL, 0, 1, varnames_for_row, vals_for_row);
 */
void EBR_milp_row_from_monomials(FILE *stream_conv,
							 int bnds_type,
							 double lb, double ub,
							 char *guard,
							 int negated,
							 unsigned num_mons,
							 char **varnames_for_row,
							 double *vals_for_row);


int EBR_milp_solve_file(char *, char *, char *, char *, unsigned);

int EBR_milp_is_exec(const char *, char *, unsigned, char *);

int EBR_milp_invoke_convexer(char *, char *, char *, unsigned);




#endif
