#include <EBR.h>
#include <Debug.h>
#include <StringUtils.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>


#define DEBUG "EBR"



static void _EBR_pastShiftAhead(EBR *_this, int timeslot);


void EBR_pastDump(EBR *_this, int timeslot)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, 1 <= timeslot && timeslot <= TIMESLOTS_IN_A_DAY, "timeslot out of bounds\n");
	Debug_assert(DEBUG, _this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL, "!(_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL)\n");

	int pastLength = _this->pastLength[timeslot-1];
	printf("*** EBR PAST (%d) ***\n", timeslot);
	if (pastLength) {
		int i, j;
		for (i = 0; i < FORECASTS_NUMBER; i++) {
			printf("_this->past[%d][%d-1][]\n", i, timeslot);
			for (j = 0; j < pastLength; j++) {
				printf("%lg ", _this->past[i][timeslot-1][j]);
			}
			printf("\n");
		}
	}
	else
		printf("*** EMPTY ***\n");
	printf("*** END ***\n");
	fflush(NULL);
}



/* Each timeslot has a stack of size TIMESLOTS_AHEAD; elements older than TIMESLOTS_AHEAD are lost forever */
void EBR_pastPush(EBR *_this, int timeslot, double consumption, double production, double PHEV)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, 1 <= timeslot && timeslot <= TIMESLOTS_IN_A_DAY, "timeslot out of bounds\n");
	Debug_assert(DEBUG, _this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL, "!(_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL)\n");

	_EBR_pastShiftAhead(_this, timeslot);
	_this->past[FORECASTS_CONSUMPTION][timeslot - 1][0] = consumption;
	if (_this->tool == TOOL_CTRL) {
		_this->past[FORECASTS_PRODUCTION][timeslot - 1][0] = production;
		_this->past[FORECASTS_PHEV][timeslot - 1][0] = PHEV;
	}
}



static void _EBR_pastShiftAhead(EBR *_this, int timeslot)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, 1 <= timeslot && timeslot <= TIMESLOTS_IN_A_DAY, "timeslot out of bounds\n");
	Debug_assert(DEBUG, _this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL, "!(_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL)\n");

	int i, j;
	/* older record is lost forever */
	for (i = 0; i < FORECASTS_NUMBER; i++) {
		for (j = _this->maxPastLength - 2; j >= 0; j--) {
			_this->past[i][timeslot - 1][j + 1] = _this->past[i][timeslot - 1][j];
		}
	}
	if (_this->pastLength[timeslot-1] < _this->maxPastLength) {
		(_this->pastLength[timeslot-1])++;
	}
}



/**
 *	Past training functions for forecasting
 */


void EBR_initPastTraining(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	if (_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL) {
		Debug_out(DEBUG, "Init Past Training (trainingForecast = 0)\n");
		_this->trainingForecast = 0;
	}
}


int EBR_isPastTraining(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	int res = 0;
	if (_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL) {
		if (_this->trainingForecast <= (_this->maxPastLength)*TIMESLOTS_IN_A_DAY) {
			res = 1;
		}
	}
	return res;
}


void EBR_nextPastTraining(EBR *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	if (_this->tool == TOOL_CTRL || _this->tool == TOOL_SUBST_CTRL) {
		if (_this->trainingForecast <= (_this->maxPastLength)*TIMESLOTS_IN_A_DAY) {
			if (_this->trainingForecast == (_this->maxPastLength)*TIMESLOTS_IN_A_DAY) {
				Debug_out(DEBUG, "*********** EXPERIMENT STARTS HERE (End future training) ***********\n");
			}
			_this->trainingForecast++;
			Debug_out(DEBUG, "Next Past Training (trainingForecast = %d)\n", _this->trainingForecast);
		}
	}
	else {
		if (_this->trainingForecast == 0) {
			Debug_out(DEBUG, "*********** EXPERIMENT STARTS HERE (End future training) ***********\n");
			_this->trainingForecast++;
		}
	}
}

