#ifndef __HECH_H__
#define __HECH_H__

#include <Array.h>
#include <stdio.h>
#include <Battery.h>




typedef enum meas {
	MEAS_ENERGY = 0,
	MEAS_CONSUMPTION,
	MEAS_PRODUCTION,
	MEAS_BATTERY,
	MEAS_PHEV,
	MEAS_PHEV_READY_HOURS,
	MEAS_NUMBER
} Measures;




typedef enum commands {
	CMDS_BATTERY = 0,
	CMDS_PHEV,
	CMDS_NUMBER
} Commands;




typedef struct HECH HECH;

#ifdef FAST_EXPS
/**
 *  Object simulating House (LabVIEW in IMDEA labs)
 *	House is a server with two channels
 *	1. From/To EBR for measurements requests
 *	2. From/To EBR for commands to battery/PHEV
 */


typedef struct House {

	int isSubstation;

	char * profilesFile;
	FILE * profilesFile_fd;
	unsigned long profilesFileCurrentLine;

	char * loggingFile;
	FILE * loggingFile_fd;

	Battery *battery;
	double batteryChargeCmd;

	Battery *PHEV;
	double phevChargeCmd;

	/* int timeSlotLength; // in minutes */
	int timeSlotLengthCtrl; // in minutes

} House;
House * House_new(Properties * conf, const char *profilesFiles, const char *loggingFile, const char * batteryFile, const char * phevFile, int isSubstation, int);
void House_free(House ** _this);
HECH * HECH_new(int controlSignalStop, House **LabView, Properties *conf, const char *profilesFile, const char *loggingFile, const char * batteryFile, const char * phevFile, int isSubstation, int);
void HECH_free(HECH **, House **);
int HECH_requestMeasurements(HECH *_this, House *, double *meas, int);
long int House_commandsFromConsumer(House *this, long int control, double * commands);
int HECH_sendCommands(HECH *_this, House *thisHouse, double *commands);
#else
HECH * HECH_new(const char * hostName, int measurementsPort, int commandsPort, int controlSignalStop);
void HECH_free(HECH **);
int HECH_requestMeasurements(HECH *_this, double *meas);
void HECH_dumpInfo(HECH *_this, FILE *stream);
int HECH_sendCommands(HECH *_this, double *commands);
#endif


int HECH_getControlStopSignal(HECH *_this);







#endif

