#define _BSD_SOURCE
#include <HECH.h>
#include <Debug.h>
#include <StringUtils.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>


#define BUFSIZE 1024
#define MAX_BUF_LEN (1 << 20)

/* To make it working on Linux we need this fake definition... */
#define h_addr h_addr_list[0]


#define DEBUG "HECH"



struct HECH {
#ifndef FAST_EXPS
	char hostName[BUFSIZE];
	int measurementsPort;
	int commandsPort;

	int sockfdMeas;
	int sockfdCmds;
#endif
	/* fields added for evaluation */
	int controlSignalStop;
	long actualControlSignal;
};



#ifndef FAST_EXPS
HECH * HECH_new(const char * hostName, int measurementsPort, int commandsPort, int controlSignalStop)
#else
HECH * HECH_new(int controlSignalStop, House **LabVIEW, Properties *conf, const char *profilesFile, const char *loggingFile, const char * batteryFile, const char * phevFile, int isSubstation, int tsl)
#endif
{
	struct sockaddr_in serv_addr_meas;
	struct sockaddr_in serv_addr_cmds;
	struct hostent *server_meas;
	struct hostent *server_cmds;

	HECH *new = (HECH *) calloc(1, sizeof(HECH));
	new->actualControlSignal = 0;
	new->controlSignalStop = controlSignalStop;
#ifndef FAST_EXPS
	sprintf(new->hostName, "%s", hostName);
	new->measurementsPort = measurementsPort;
	new->commandsPort = commandsPort;

	/** Connects to House server */

	/* Measurements */
	new->sockfdMeas = socket(AF_INET, SOCK_STREAM, 0);
	if (new->sockfdMeas < 0) {
		Debug_out(DEBUG, "ERROR opening socket for requesting measurements\n");
		return NULL;
	}
	server_meas = gethostbyname(hostName);
	if (server_meas == NULL) {
		Debug_out(DEBUG, "ERROR, no such host %s\n", hostName);
		return NULL;
	}
	bzero((char *) &serv_addr_meas, sizeof(serv_addr_meas));
	serv_addr_meas.sin_family = AF_INET;
	bcopy((char *)server_meas->h_addr,
		  (char *)&serv_addr_meas.sin_addr.s_addr,
		  server_meas->h_length);
	serv_addr_meas.sin_port = htons(new->measurementsPort);
	/* Now connect to the server */
	if (connect(new->sockfdMeas, (struct sockaddr *) &serv_addr_meas, sizeof(serv_addr_meas)) < 0) {
		Debug_out(DEBUG, "ERROR connecting to House server for requesting measurements at %s:%d\n", hostName, new->measurementsPort);
		return NULL;
	}
	Debug_out(DEBUG, "Connected to %s:%d for measurement requests\n", hostName, new->measurementsPort);

	/* Commands */
	new->sockfdCmds = socket(AF_INET, SOCK_STREAM, 0);
	if (new->sockfdCmds < 0) {
		Debug_out(DEBUG, "ERROR opening socket for commands\n");
		return NULL;
	}
	server_cmds = gethostbyname(hostName);
	if (server_cmds == NULL) {
		Debug_out(DEBUG, "ERROR, no such host %s\n", hostName);
		return NULL;
	}
	bzero((char *) &serv_addr_cmds, sizeof(serv_addr_cmds));
	serv_addr_cmds.sin_family = AF_INET;
	bcopy((char *)server_cmds->h_addr,
		  (char *)&serv_addr_cmds.sin_addr.s_addr,
		  server_cmds->h_length);
	serv_addr_cmds.sin_port = htons(new->commandsPort);
	/* Now connect to the server */
	if (connect(new->sockfdCmds, (struct sockaddr *) &serv_addr_cmds, sizeof(serv_addr_cmds)) < 0) {
		Debug_out(DEBUG, "ERROR connecting to House server for commands at %s:%d\n", hostName, new->commandsPort);
		return NULL;
	}
	Debug_out(DEBUG, "Connected to %s:%d for commands\n", hostName, new->commandsPort);
#else
	*LabVIEW = House_new(conf, profilesFile, loggingFile, batteryFile, phevFile, isSubstation, tsl);
	if (*LabVIEW == NULL) {
		fprintf(stderr, "Problems in initializing readings from file\n");
		exit(1);
	}

#endif
	
	return new;
}

#ifdef FAST_EXPS
void House_free(House ** _this)
{
	Debug_assert(DEBUG, *_this != NULL, "*_this == NULL\n");
	fclose((*_this)->profilesFile_fd);
	fclose((*_this)->loggingFile_fd);
	if ((*_this)->battery) {
		Battery_free(&(*_this)->battery);
	}
	if ((*_this)->PHEV) {
		Battery_free(&(*_this)->PHEV);
	}
	free(*_this);
	*_this = NULL;
}

House * House_new(Properties *conf, const char *profilesFile, const char *loggingFile, const char * batteryFile, const char * phevFile, int isSubstation, int tsl)
{
	House * new = (House *) calloc(1, sizeof(House));
	int controlSignalStop;
	static char line[MAX_BUF_LEN];

	if (Properties_getInt(conf, "controlSignalStop", &controlSignalStop) < 0) {
		fprintf(stderr, "\nAdd in configuration file a line\n\nint controlSignalStop = anInteger\n");
		exit(1);
	}

	/* if (Properties_getInt(conf, "timeSlotLength", &(new->timeSlotLength)) < 0) { */
	/* 	fprintf(stderr, "\nAdd in configuration file a line\n\nint timeSlotLength = anIntegerInMinutes\n"); */
	/* 	exit(1); */
	/* } */

	/* if (Properties_getInt(conf, "timeSlotCtrlLength", &(new->timeSlotLengthCtrl)) < 0) { */
	/* 	fprintf(stderr, "\nAdd in configuration file a line\n\nint timeSlotCtrlLength = anIntegerInMinutes\n"); */
	/* 	exit(1); */
	/* } */

	new->timeSlotLengthCtrl = tsl;

	new->isSubstation = isSubstation;


	/** Open profiles input files and logging output file */
	new->profilesFile_fd = fopen(profilesFile, "r");
	if (!new->profilesFile_fd) {
		fprintf(stderr, "Error opening file %s\n", profilesFile);
		exit(1);
	}
	if (!fgets(line, MAX_BUF_LEN, new->profilesFile_fd)) {
		fprintf(stderr, "Problems reading from %s\n", new->profilesFile);
		exit(1);
	}
	if (line[strlen(line)-1] == '\n') {
		line[strlen(line)-1] = '\0';
	}
	Debug_out(DEBUG, "Profiles first line : %s", line);

	new->loggingFile_fd = fopen(loggingFile, "w");
	if (!new->loggingFile_fd) {
		fprintf(stderr, "Error creating file %s\n", loggingFile);
		exit(1);
	}
	if (new->isSubstation || !phevFile) {
		fprintf(new->loggingFile_fd, "control,starting,ending,energy_grid,demand,battery,battery_charge\n");
	}
	else {
		fprintf(new->loggingFile_fd, "control,starting,ending,energy_grid,consumption,production,battery,phev,battery_charge,phev_charge,phev_ready_hours\n");
	}
	fflush(new->loggingFile_fd);

	/* read battery and PHEV properties, initially set to half their capacity */
	new->battery = Battery_new(batteryFile);
	Debug_out(DEBUG, "Battery:\n");
	/* if (!infESSCapacity) */
		Battery_setStateOfCharge(new->battery, Battery_getCapacity(new->battery)/2);
	/* else */
	/* 	Battery_setStateOfCharge(new->battery, 0); */
	Debug_perform(DEBUG, Battery_dump(new->battery, stdout));

	if (!new->isSubstation && phevFile) {
		new->PHEV = Battery_new(phevFile);
		Debug_out(DEBUG, "PHEV:\n");
		Battery_setStateOfCharge(new->PHEV, -1);
		Debug_perform(DEBUG, Battery_dump(new->PHEV, stdout));
	}
	else {
		new->PHEV = NULL;
		Debug_out(DEBUG, "PHEV is NULL\n");
	}

	new->batteryChargeCmd = 0;
	new->phevChargeCmd = 0;

	return new;
}

long int House_commandsFromConsumer(House *this, long int control, double * commands)
{
	Debug_assert(DEBUG, this != NULL, "this == NULL\n");

	/* control signal */
	if (control != this->profilesFileCurrentLine) {
		Debug_out(DEBUG, "Commanding with control '%ld' while actual control is '%ld'\n", control, this->profilesFileCurrentLine);
		return -2;
	}

	/* battery */
	this->batteryChargeCmd = commands[CMDS_BATTERY];

	/* phev */
	if (this->isSubstation || !this->PHEV) {
		this->phevChargeCmd = 0;
	}
	else {
		this->phevChargeCmd = commands[CMDS_PHEV];
	}

	Debug_out(DEBUG, "Command '%ld %lg %lg' registered\n", control, commands[CMDS_BATTERY], commands[CMDS_PHEV]);

	/* ack to command */
	return 0;
}

/**
 *	- If there are no more data to be applied, 
		the House server (LabVIEW) sends
 (1)<- -1

 - If there was an error that obligates to stop the experiment
 (1)<- -2
 */
long int House_measurementsReqsFromConsumer(House *this, long int control, double * measurements, int readFromFile)
{
	Debug_assert(DEBUG, this != NULL, "this == NULL\n");

	static char line[MAX_BUF_LEN];

	char *lineTmp;
	ssize_t nbytesread;
	char * token;
	char * starting;
	char * ending;
	double energyGrid = 0;
	double phev = -1;

	Debug_out(DEBUG, "Received control '%ld'\n", control);
	if (control != (this->profilesFileCurrentLine) + 1) {
		Debug_out(DEBUG, "Requesting control '%ld' while last requested control was '%ld'\n", control, this->profilesFileCurrentLine);
		return -2;
	}
	Debug_out(DEBUG, "Requested control '%ld'\n", control);

	/* Increase control signal */
	this->profilesFileCurrentLine = (this->profilesFileCurrentLine) + 1;

	if (readFromFile) {
	  /* Read current line from file and set measurements accordingly */
	  if (!fgets(line, MAX_BUF_LEN, this->profilesFile_fd)) {
	    Debug_out(DEBUG, "No more lines to read from profiles file %s\n", this->profilesFile);
	    return -1;
	  }
	  if (line[strlen(line)-1] == '\n') {
	    line[strlen(line)-1] = '\0';
	  }
	  Debug_out(DEBUG, "Retrieved profiles : %s\n", line);
	} /* it is static, should continue with old value */

	/* starting */
	lineTmp = StringUtils_clone(line);
	token = strsep(&lineTmp, ",");
	if (!token) {
		perror("Problems tokenizing file line");
		return -2;
	}
	Debug_out(DEBUG, "token = '%s'\n", token);
	starting = StringUtils_clone(token);
	/* ending */
	token = strsep(&lineTmp, ",");
	if (!token) {
		perror("Problems tokenizing file line");
		return -2;
	}
	Debug_out(DEBUG, "token = '%s'\n", token);
	ending = StringUtils_clone(token);
	/* consumption */
	token = strsep(&lineTmp, ",");
	if (!token) {
		perror("Problems tokenizing file line");
		return -2;
	}
	Debug_out(DEBUG, "token = '%s'\n", token);
	if (StringUtils_toDouble(token, &measurements[MEAS_CONSUMPTION]) != 0) {
		Debug_out(DEBUG, "Problems reading consumption, value '%s' not accepted\n", token);
		return -2;
	}
	if (!this->isSubstation) {
		/* production */
		token = strsep(&lineTmp, ",");
		if (!token) {
			perror("Problems tokenizing file line");
			return -2;
		}
		Debug_out(DEBUG, "token = '%s'\n", token);
		if (StringUtils_toDouble(token, &measurements[MEAS_PRODUCTION]) != 0) {
			Debug_out(DEBUG, "Problems reading production, value '%s' not accepted\n", token);
			return -2;
		}
		/* check phev presence */
		if (this->PHEV) {
			token = strsep(&lineTmp, ",");
			if (!token) {
				perror("Problems tokenizing file line");
				return -2;
			}
			Debug_out(DEBUG, "token = '%s'\n", token);
			if (!StringUtils_equals(token, "")) {
				if (StringUtils_toDouble(token, &phev) != 0) {
					Debug_out(DEBUG, "Problems reading phev, value %s not accepted\n", token);
					return -2;
				}
				Debug_out(DEBUG, "phev = '%lg'\n", phev);
				/* if not present before, register soc of plugging PHEV */
				/* always set if in simulation */
				if (phev != -1 && Battery_getStateOfCharge(this->PHEV) == -1) {
					Battery_setStateOfCharge(this->PHEV, phev);
				}
			}
			else {
				Debug_out(DEBUG, "PHEV not present\n");
				Battery_setStateOfCharge(this->PHEV, -1);
			}
			/* phev charging kwh */
			token = strsep(&lineTmp, ",");
			if (!token) {
				perror("Problems tokenizing file line");
				return -2;
			}
			Debug_out(DEBUG, "token = '%s'\n", token);
			/* phev user desiderata: hours in which it must be ready */
			token = strsep(&lineTmp, ",");
			if (!token) {
				perror("Problems tokenizing file line");
				return -2;
			}
			Debug_out(DEBUG, "token = '%s'\n", token);
			if (StringUtils_toDouble(token, &measurements[MEAS_PHEV_READY_HOURS]) != 0) {
				Debug_out(DEBUG, "Problems reading production, value '%s' not accepted\n", token);
				return -2;
			}
		}
	}
	else { /* in case of Substation, just to be sure, zero every non related meas */
		measurements[MEAS_PRODUCTION] = 0;
		measurements[MEAS_PHEV] = 0;
		measurements[MEAS_PHEV_READY_HOURS] = 0;
	}
	free(lineTmp);


	{
		double factor = this->battery && this->batteryChargeCmd < 0? Battery_getDischargeFactor(this->battery) : 1;
		/* Charge or discharge battery as commanded, only if not in simulation */
		if (this->battery && this->batteryChargeCmd != 0) {
			Battery_actuate(this->battery, this->batteryChargeCmd, (double)this->timeSlotLengthCtrl/60.0);
		}
		double factorPhev = this->PHEV && this->phevChargeCmd < 0? Battery_getDischargeFactor(this->PHEV) : 1;
		if (!this->isSubstation && this->PHEV && this->phevChargeCmd > 0 && Battery_getStateOfCharge(this->PHEV) > -1) {
			Battery_actuate(this->PHEV, this->phevChargeCmd, (double)this->timeSlotLengthCtrl/60.0);
		}
		energyGrid = measurements[MEAS_CONSUMPTION] + factor * this->batteryChargeCmd + factorPhev * this->phevChargeCmd - measurements[MEAS_PRODUCTION];
	}

	/* Set remaining measurements fields */
	measurements[MEAS_ENERGY] = energyGrid;
	if (this->battery) {
		measurements[MEAS_BATTERY] = Battery_getStateOfCharge(this->battery);
	}
	else {
		measurements[MEAS_BATTERY] = 0;
	}
	if (!this->isSubstation && this->PHEV) {
		measurements[MEAS_PHEV] = Battery_getStateOfCharge(this->PHEV);
	}

	/* dump loggingFile in CSV */
	if (this->isSubstation || !this->PHEV) {
		fprintf(this->loggingFile_fd, "%lu,%s,%s,%lg,%lg,%lg,%lg\n", this->profilesFileCurrentLine, starting, ending, measurements[MEAS_ENERGY], measurements[MEAS_CONSUMPTION], measurements[MEAS_BATTERY], this->batteryChargeCmd);
	}
	else {
		if (Battery_getStateOfCharge(this->PHEV) == -1) {
			fprintf(this->loggingFile_fd, "%lu,%s,%s,%lg,%lg,%lg,%lg,,%lg,,\n", this->profilesFileCurrentLine, starting, ending, measurements[MEAS_ENERGY], measurements[MEAS_CONSUMPTION], measurements[MEAS_PRODUCTION], measurements[MEAS_BATTERY], this->batteryChargeCmd);
		}
		else {
			fprintf(this->loggingFile_fd, "%lu,%s,%s,%lg,%lg,%lg,%lg,%lg,%lg,%lg,%lg\n", this->profilesFileCurrentLine, starting, ending, measurements[MEAS_ENERGY], measurements[MEAS_CONSUMPTION], measurements[MEAS_PRODUCTION], measurements[MEAS_BATTERY], measurements[MEAS_PHEV], this->batteryChargeCmd, this->phevChargeCmd, measurements[MEAS_PHEV_READY_HOURS]);
		}
	}
	fflush(this->loggingFile_fd);

	return this->profilesFileCurrentLine;
}
#endif

#ifndef FAST_EXPS
void HECH_free(HECH **_this)
#else
void HECH_free(HECH **_this, House **thisHouse)
#endif
{
	Debug_assert(DEBUG, *_this != NULL, "*_this == NULL\n");
	/* Disconnects from House server */
	HECH * this = *_this;
#ifndef FAST_EXPS
	Debug_out(DEBUG, "Closing connection with %s:%d\n", this->hostName, this->measurementsPort);
	Debug_out(DEBUG, "Closing connection with %s:%d\n", this->hostName, this->commandsPort);
	close(this->sockfdMeas);
	close(this->sockfdCmds);
#else
	House_free(thisHouse);
#endif
	free(this);
	*_this = NULL;
}


int HECH_getControlStopSignal(HECH *_this)
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	return _this->controlSignalStop;
}



#ifndef FAST_EXPS
int HECH_requestMeasurements(HECH *_this, double *meas)
#else
  int HECH_requestMeasurements(HECH *_this, House *thisHouse, double *meas, int readFromFile)
#endif
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, meas != NULL, "meas == NULL\n");

	int n, i;
	static long int readControl;

	/* Updates control signal */
	_this->actualControlSignal = _this->actualControlSignal + 1;

#ifndef FAST_EXPS
	/* Send message to the server */
	n = send(_this->sockfdMeas, (char *)&(_this->actualControlSignal), sizeof(long int), 0);
	if (n <= 0) {
		Debug_out(DEBUG, "ERROR sending measurements request '%ld'\n", _this->actualControlSignal);
		return -2;
	}
	Debug_out(DEBUG, "Sent message '%ld'\n", _this->actualControlSignal);

	/* Read server response */
	n = recv(_this->sockfdMeas, (char *)&readControl, sizeof(long int), 0);
	if (n < 0) {
		Debug_out(DEBUG, "ERROR reading from socket while receiving measurements\n");
		return -2;
	}
	else if (n == 0) {
		Debug_out(DEBUG, "ERROR server closed its connection\n");
		return -2;
	}
	/* stop if need be */
	if (readControl == _this->controlSignalStop || readControl == -2) {
		return readControl;
	}
	Debug_out(DEBUG, "readControl = %ld\n", readControl);

	/* Fill parameter meas with answer from server */
	for (i = 0; i < MEAS_NUMBER; i++) {
		n = recv(_this->sockfdMeas, (char *)&meas[i], sizeof(double), 0);
		if (n < 0) {
			Debug_out(DEBUG, "ERROR reading from socket while receiving measurements\n");
			return -2;
		}
		else if (n == 0) {
			Debug_out(DEBUG, "ERROR server closed its connection\n");
			return -2;
		}
		Debug_out(DEBUG, "meas[%d] = %lg\n", i, meas[i]);
	}
#else
	readControl = House_measurementsReqsFromConsumer(thisHouse, _this->actualControlSignal, meas, readFromFile);
	/* if (parsingRes == -2) { */
	/*   fprintf(stderr, "ERROR in reading measurements at control %ld\n", _this->actualControlSignal); */
	/*   break; */
	/* } */
	/* else if (parsingRes == -1) { */
	/*   fprintf(stderr, "Terminated profiles reading\n"); */
	/*   break; */
	/* } */
#endif
	return readControl;
}


#ifndef FAST_EXPS
int HECH_sendCommands(HECH *_this, double *commands)
#else
int HECH_sendCommands(HECH *_this, House *thisHouse, double *commands)
#endif
{
	Debug_assert(DEBUG, _this != NULL, "_this == NULL\n");
	Debug_assert(DEBUG, commands != NULL, "commands == NULL\n");

	int n, i;
	long int readControl = 0;

#ifndef FAST_EXPS
	/* Send message to the server */
	n = send(_this->sockfdCmds, (char *)&(_this->actualControlSignal), sizeof(long int), 0);
	if (n <= 0) {
		Debug_out(DEBUG, "ERROR sending commands '%ld'\n", _this->actualControlSignal);
		return -2;
	}
	for (i = 0; i < CMDS_NUMBER; i++) {
		n = send(_this->sockfdCmds, (char *)&commands[i], sizeof(double), 0);
		if (n <= 0) {
			Debug_out(DEBUG, "ERROR sending commands '%lg'\n", commands[i]);
			return -2;
		}
	}
	Debug_out(DEBUG, "Sent command '%ld %lg %lg'\n", _this->actualControlSignal, commands[CMDS_BATTERY], commands[CMDS_PHEV]);

	/* Read server response */
	n = recv(_this->sockfdCmds, (char *)&readControl, sizeof(long int), 0);
	if (n < 0) {
		Debug_out(DEBUG, "ERROR reading commands ack\n");
		return -2;
	}
	else if (n == 0) {
		Debug_out(DEBUG, "ERROR server closed its connection\n");
		return -2;
	}
	Debug_out(DEBUG, "Read message '%ld'\n", readControl);
#else
	readControl = House_commandsFromConsumer(thisHouse, thisHouse->profilesFileCurrentLine, commands);
#endif
	return readControl;
}


#ifndef FAST_EXPS
void HECH_dumpInfo(HECH *_this, FILE *stream)
{
	Debug_assert(DEBUG, _this != NULL, "*_this == NULL\n");

	fprintf(stream, "==== HECH Information ==== (%p)\n", _this);
	fprintf(stream, "host name:\t%s\n", _this->hostName);
	fprintf(stream, "port for measurements requests:\t%d\n", _this->measurementsPort);
	fprintf(stream, "port for commands:\t%d\n", _this->commandsPort);
	fprintf(stream, "control signal stop:\t%d\n", _this->controlSignalStop);
	fprintf(stream, "==========================\n");
	fflush(stream);
}
#endif
