#include <HECH.h>
#include <Debug.h>
#include <Array.h>


#define DEBUG "TestHECH"


int main(int argc, char **argv)
{
	HECH * myHECH = NULL;
	double measurements[MEAS_NUMBER];
	double commands[CMDS_NUMBER];
	int answer;

	/* Connect to the House */
	myHECH = HECH_new("127.0.0.1", 2324, 2325, -1);
	if (!myHECH) {
		Debug_out(DEBUG, "Failed to connect to the House server\n");
		return 1;
	}

	/* Read measurements from the House */
	Debug_out(DEBUG, "Read measurements from House server\n");

	answer = HECH_requestMeasurements(myHECH, measurements);
	if (answer == HECH_getControlStopSignal(myHECH)) {
		Debug_out(DEBUG, "Control STOP signal sent from House server, answer = %d\n", answer);
		return 1;
	}
	else if (answer == -2) {
		Debug_out(DEBUG, "Cannot get measurements from House server, answer = %d\n", answer);
		return 1;
	}
	Debug_out(DEBUG, "Measurements from House : energy = %lg, consumption = %lg, production = %lg, battery SoC = %lg, PHEV SoC = %lg\n", measurements[MEAS_ENERGY], measurements[MEAS_CONSUMPTION], measurements[MEAS_PRODUCTION], measurements[MEAS_BATTERY], measurements[MEAS_PHEV]);

	commands[CMDS_BATTERY] = 0;
	commands[CMDS_PHEV] = 0;

	/* Send commands to the House */
	Debug_out(DEBUG, "Send commands to House server\n");

	answer = HECH_sendCommands(myHECH, commands);
	if (answer < 0) {
		Debug_out(DEBUG, "Commands not accepted by server, answer = %d\n", answer);
	}
	Debug_out(DEBUG, "Sent commands to House : battery = %lg, PHEV = %lg\n", commands[CMDS_BATTERY], commands[CMDS_PHEV]);

	HECH_free(&myHECH);

	return 0;
}
