#ifndef __COMMAND_LINE_INTERFACE__
#define __COMMAND_LINE_INTERFACE__

#include <Properties.h>

typedef struct CommandLineInterface CommandLineInterface;

CommandLineInterface* CommandLineInterface_new(const char* const args0, const char* const syntaxHeader);

/** Returns a const Properties* object with all given CLI options, 
	including default values for options not given
*/	
const Properties* const CommandLineInterface_args(const CommandLineInterface* const _this);

void CommandLineInterface_addDoubleOption(CommandLineInterface* this, const char* const option, const char* const help);
void CommandLineInterface_addDoubleOption_withDefault(CommandLineInterface* this, const char* const option, double default_value, const char* const help);

void CommandLineInterface_addIntOption(CommandLineInterface* this, const char* const option, const char* const help);
void CommandLineInterface_addIntOption_withDefault(CommandLineInterface* this, const char* const option, int default_value, const char* const help);

void CommandLineInterface_addULongOption(CommandLineInterface* this, const char* const option, const char* const help);
void CommandLineInterface_addULongOption_withDefault(CommandLineInterface* this, const char* const option, unsigned long default_value, const char* const help);

void CommandLineInterface_addStringOption(CommandLineInterface* this, const char* const option, const char* const help);
void CommandLineInterface_addStringOption_withDefault(CommandLineInterface* this, const char* const option, const char* const default_value, const char* const help);


void CommandLineInterface_fprint_args(const CommandLineInterface* const _this, FILE* f);

void CommandLineInterface_addFlag(CommandLineInterface* _this, const char* const flag, const char* const help);

// Processes command-line args. Returns 0 in case of success and an error code (int < 0) otherwise.
int CommandLineInterface_processArgs(CommandLineInterface* _this, int argc, char** args);

void CommandLineInterface_free(CommandLineInterface** thisP);


const char* const CommandLineInterface_getStringOption(const CommandLineInterface* const _this, char* property);
int CommandLineInterface_getIntOption(const CommandLineInterface* const _this, char* property);
unsigned long CommandLineInterface_getULongOption(const CommandLineInterface* const _this, char* property);
double CommandLineInterface_getDoubleOption(const CommandLineInterface* const _this, char* property);
int CommandLineInterface_getFlag(const CommandLineInterface* const _this, char* property);
const char* const CommandLineInterface_getFullCmdline(const CommandLineInterface* const _this);

const char* const CommandLineInterface_getWorkingDir(const CommandLineInterface* const _this);
#endif
