#ifndef __RND_GEN_DISTRIBUTION__
#define __RND_GEN_DISTRIBUTION__

#include <RndGen.h>


typedef struct RndGenDistribution RndGenDistribution;

RndGenDistribution* RndGenDistribution_new(RndGen* rnd, unsigned long x_min, unsigned long x_max, int decimal_places);

RndGenDistribution* RndGenDistribution_new_fromFunction(
	RndGen* rnd, double (*probability)(unsigned long), unsigned long x_min, unsigned long x_max, 
	int decimal_places);
	
	/* meaningful decimal places (precision: the higher the better)
	   Each value of probability() is approximated as a multiple of 1/(10^decimal_places).
	
	  Example: assume that x_min = 0, x_max = 3 and probability() = {1-->0.1, 2-->0.73, 3-->0.26}
	  If decimal_places = 1, we are scaling the probability distribution as follows:
	      x=1 --> (int) 0.1  * 10^1 = 1
	      x=2 --> (int) 0.73 * 10^1 = 7
	      x=3 --> (int) 0.26 * 10^1 = 2
	
	  With a better precision, e.g., decimal_places = 2, we are scaling the probability distribution as follows:
		x=1 --> (int) 0.1  * 10^2 = 10
		x=2 --> (int) 0.73 * 10^2 = 73
		x=3 --> (int) 0.26 * 10^2 = 26
	*/

void RndGenDistribution_set(RndGenDistribution* _this, unsigned long x, double p);
double RndGenDistribution_get(RndGenDistribution* _this, unsigned long x);

unsigned long RndGenDistribution_domain_min(const RndGenDistribution* const _this);
unsigned long RndGenDistribution_domain_max(const RndGenDistribution* const _this);


RndGen* RndGenDistribution_getRndGen(RndGenDistribution* _this);
unsigned long RndGenDistribution_next(RndGenDistribution* _this);

void RndGenDistribution_free(RndGenDistribution** thisP);

#endif
