#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>

#include <Debug.h>
#include <Properties.h>


static Properties* categoriesOut = NULL;
static Properties* categoriesAssertions = NULL;

static int defaultValueOut = 1; // categories not occurring in 'categoriesOut'
static int defaultValueAssertions = 1; // categories not occurring in 'categoriesAssertions'

 
void Debug_setDefaultOut(int d) {
	defaultValueOut = d;
	Debug_out(DEBUG_ALWAYS, "Debug: debug output for all categories %s by default\n", 
	 	(d ? "enabled" : "disabled"));
}
void Debug_setDefaultAssertions(int d) {
	defaultValueAssertions = d;
	Debug_out(DEBUG_ALWAYS, "Debug: assertion checking for all categories %s by default\n", 
	 	(d ? "enabled" : "disabled"));
}


int Debug_isEnabled(const char* const id, int WHAT) {
	//fprintf(stderr, "Debug_isEnabled('%s', %d) start\n", id, WHAT);
	//fprintf(stderr, " - checking whether '%s' equals '%s': ", id, DEBUG_ALWAYS);	
	if (strcmp(id, DEBUG_ALWAYS) == 0) {
		//fprintf(stderr, "yes --> returning 1\n");
		return 1;
	}
	//fprintf(stderr, "no\n");	
	int value = 0;
	Properties* p = (WHAT == _OUT) ? categoriesOut : categoriesAssertions;
	int error = 0;
	if (p == NULL) {
		error = 1;
		//fprintf(stderr, " - Properties == NULL --> error = %d\n", error);		
	}
	else {
		//fprintf(stderr, " - getting value from Properties %p...", (void*)p);
		error = Properties_getBoolean(p, id, &value);
		//fprintf(stderr, " error = %d\n", error);
	}
	if (error) {
		//fprintf(stderr, " - error! --> returning defaultValue\n");		
		return (WHAT == _OUT) ? defaultValueOut : defaultValueAssertions;
	}	
	else {
		//fprintf(stderr, " - returning %d\n", value);
		return value;
	}
}

void Debug_enableOut(const char* const id) {
	Debug_assert(DEBUG_ALWAYS, strcmp(id, DEBUG_ALWAYS) != 0, "You cannot change the behaviour of the special Debug category '%s'\n", DEBUG_ALWAYS);
	if (categoriesOut == NULL) categoriesOut = Properties_new(CATEGORY_MAX_LENGTH, CATEGORY_MAX_LENGTH);	
	Debug_assert(DEBUG_ALWAYS, categoriesOut != NULL, "Could not allocate categoriesOut\n");	
	Properties_setBoolean(categoriesOut, id, 1);
	Debug_out(DEBUG_ALWAYS, "Debug: debug output for category '%s' enabled\n", id);
}
void Debug_enableAssertions(const char* const id) {
	Debug_assert(DEBUG_ALWAYS, strcmp(id, DEBUG_ALWAYS) != 0, "You cannot change the behaviour of the special Debug category '%s'\n", DEBUG_ALWAYS);
	//fprintf(stderr, "0 - cat:%p, id:'%s':%p\n", (void*)categoriesAssertions, id, (void*)id);	
	if (categoriesAssertions == NULL) categoriesAssertions = Properties_new(CATEGORY_MAX_LENGTH, CATEGORY_MAX_LENGTH);		
	Debug_assert(DEBUG_ALWAYS, categoriesAssertions != NULL, "Could not allocate categoriesAssertions\n");
	//fprintf(stderr, "1 - %p\n", (void*)categoriesAssertions);
	Properties_setBoolean(categoriesAssertions, id, 1);
	//fprintf(stderr, "2 - %p\n", (void*)categoriesAssertions);	
	Debug_out(DEBUG_ALWAYS, "Debug: assertion checking for category '%s' enabled\n", id);	
}

void Debug_disableOut(const char* const id) {	
	Debug_assert(DEBUG_ALWAYS, strcmp(id, DEBUG_ALWAYS) != 0, "You cannot change the behaviour of the special Debug category '%s'\n", DEBUG_ALWAYS);
	if (categoriesOut == NULL) categoriesOut = Properties_new(CATEGORY_MAX_LENGTH, CATEGORY_MAX_LENGTH);	
	Debug_assert(DEBUG_ALWAYS, categoriesOut != NULL, "Could not allocate categoriesOut\n");		
	Properties_setBoolean(categoriesOut, id, 0);
	Debug_out(DEBUG_ALWAYS, "Debug: debug output for category '%s' disabled\n", id);	
}
void Debug_disableAssertions(const char* const id) {
	Debug_assert(DEBUG_ALWAYS, strcmp(id, DEBUG_ALWAYS) != 0, "You cannot change the behaviour of the special Debug category '%s'\n", DEBUG_ALWAYS);
	if (categoriesAssertions == NULL) categoriesAssertions = Properties_new(CATEGORY_MAX_LENGTH, CATEGORY_MAX_LENGTH);		
	Debug_assert(DEBUG_ALWAYS, categoriesAssertions != NULL, "Could not allocate categoriesAssertions\n");
	Properties_setBoolean(categoriesAssertions, id, 0);
	Debug_out(DEBUG_ALWAYS, "Debug: assertion checking for category '%s' disabled\n", id);		
}

void Debug_enable(const char* const id) {
	Debug_enableOut(id);
	Debug_enableAssertions(id);
}
void Debug_disable(const char* const id) {
	Debug_disableOut(id);
	Debug_disableAssertions(id);
}



void Debug_freeAll() {
	Properties_free(&categoriesOut);
	Properties_free(&categoriesAssertions);
}


unsigned long long Debug_timestamp_millisec() {
	struct timeval tv;
	gettimeofday(&tv, NULL);
	return ((unsigned long long)tv.tv_sec) * 1000 + 
		(((unsigned long long)tv.tv_usec) / 1000);
}

