#include <RndGen.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <gmp.h>


// Implemented using GMP
struct RndGen {
	gmp_randstate_t state;
	unsigned long seed;
};


// Create a new random generator (with seed = time(NULL))
RndGen* RndGen_new() {
	RndGen* new = (RndGen*)malloc(sizeof(RndGen));
	gmp_randinit_default(new->state);
	RndGen_setSeed(new, time(NULL) );	
	return new;
}
void RndGen_free(RndGen** rndPtr) {
	RndGen* rnd = *rndPtr;
	gmp_randclear(rnd->state);
	free(rnd);
	*rndPtr = NULL;
}

// (Re-)set seed
void RndGen_setSeed(RndGen* rnd, unsigned long int seed) {
	rnd->seed = seed;
	gmp_randseed_ui (rnd->state, rnd->seed);	
}
unsigned long RndGen_getSeed(RndGen* rnd) {
	return rnd->seed;
}


// Get next number as unsigned long between from and to (inclusive)
unsigned long RndGen_nextUL(RndGen* rnd, unsigned long from, unsigned long to) {
	return gmp_urandomm_ui(rnd->state, to-from+1) + from;
}

// Get next number as double between from and to
double RndGen_nextD(RndGen* rnd, double from, double to) {
	fprintf(stderr, "RndGen_nextD() not yet implemented.");
	exit(-1);
}

double RndGen_nextDQuantized(RndGen *rnd, double from, double to, int parts) {
	if (parts <= 1) {
		fprintf(stderr, "RndGen_nextDQuantized(): parts must be > 1. Now = %d\n", parts);
		exit(-1);
	}
	int max_p = parts - 1;
	unsigned long  x = RndGen_nextUL(rnd, 0, max_p);
	return from + x * (to - from)/max_p;
}



