#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <RndGen.h>
#include <RndGenDistribution.h>
#include <Debug.h>
#include <Array.h>

double probability(unsigned long x) {
	switch (x) {
		case 0 : return 0;
		
		case 1 : return 0;
		case 2 : return 0.5;
		case 3 : return 0.5;
		case 4 : return 1;
		case 5 : return 0;
		case 6 : return 0;										
		default : return 0;
	}
	return 0;
}

void RndGen_test() {
	RndGen* rnd = RndGen_new();
	RndGen_setSeed(rnd, time(NULL) );
	unsigned long x_min = 5;
	unsigned long x_max = 100;	
	int tries = 50;
	printf("\nUniformly extracting %d integers in %lu..%lu (with RndGen_nextUL):\n", 
		tries, x_min, x_max);	
	for(int i=0; i<tries; i++) {
		printf(" - %lu\n", RndGen_nextUL(rnd, 5, 100));
	}
	printf("end\n\n");
	
	int dec_places = 1;
	x_min = 1;
	x_max = 6;	
	fprintf(stderr, "Creating distribution...\n");		
	RndGenDistribution* distr = RndGenDistribution_new_fromFunction(rnd, probability, x_min, x_max, dec_places);
	if (distr == NULL) {
		fprintf(stderr, "DISTRIBUTION NULL!\n");
		exit(EXIT_FAILURE);
	}
	fprintf(stderr, "\n\n");
			
	tries = 100000;		
	printf("Distribution:\n");	
	for(unsigned long x=x_min; x <= x_max; x++) {
		double p = RndGenDistribution_get(distr, x);
		printf(" - probability(%lu) = %lf (%E)\n", x, p, p);
	}
	printf("\n");
	
	Array* occurrences = Array_newInt(x_max - x_min + 1, x_max - x_min + 1);
	int zero = 0;
	for(int x = x_min; x <= x_max; x++) {
		Array_add(occurrences, &zero);
	}
	
	printf("Performing %d extractions according to above distribution:\n", tries);
	for(int i=0; i<tries; i++) {
		unsigned long x = RndGenDistribution_next(distr);
		double p = RndGenDistribution_get(distr, x);
		/*printf("  [%d] x=%lu (probability = %lf)\n", 
			i, x, p);*/
		int occ = 0;
		Array_get(occurrences, x - x_min, &occ);
		occ++;
		Array_set(occurrences, x - x_min, &occ);
	}
	printf("done\n\n");
	
	printf("Frequencies of values vs. their probability\n");
	for(int x = x_min; x <= x_max; x++) {
		int occ = 0;
		Array_get(occurrences, x-x_min, &occ);
		printf(" x=%d: frequency = %lf, probability = %lf\n",
			x, ((double)occ)/tries,  RndGenDistribution_get(distr, x) );
	}
	printf("end\n\n");
	
	
	
	RndGenDistribution_free(&distr);
	
	tries = 50;
	printf("Testing RngGen_RndGen_nextDQuantized() for %d tries:\n\n", tries);
	for(int i=0; i<tries; i++) {
		printf(" - %lf\n", RndGen_nextDQuantized(rnd, 0, 9, 9));
	}
	printf("end\n\n");
	
	
	
	RndGen_free(&rnd);
}


int main() {
	Debug_setDefaultOut(0);
	Debug_enableOut("RndGenDistribution");
	RndGen_test();
}

