#! /usr/bin/env python

from __future__ import with_statement
from optparse import OptionParser

import sys
import os
import pdb

from convexer3.printer.printing import Printer

# global variables: options, args

# command line parsing
def parse_command_line():
    global options, args
    parser = OptionParser()
    parser.add_option("-f", "--file", dest="inputfilename",
                      help="read model from FILE", metavar="FILE")
    parser.add_option("-o", "--output", dest="outputfilename",
                      help="write report to FILE", metavar="FILE")
    parser.add_option("-n", "--no-convex", dest="convex",
                      action="store_false", help="do not perform convexification (maintain implications); with this options variables' bounds can be specified in -infty .. +infty",
                      default=True)    
    parser.add_option("-p", "--prefix", dest="prefix_of_new_vars",
                      help="prefix of new variables added by translation of ORs", default="z_z_")
    parser.add_option("-s", "--num", dest="varcounter",
                      help="starting number (in suffix) of new variables added by translation of ORs", 
                      default=0)
    parser.add_option("-v", "--verbose", dest="verbose",
                      action="store_true", help="show information on exec",
                      default=False)    
    parser.add_option("-a", "--auxiliary", dest="dumpAuxiliary",
                      action="store_true", help="write \"Auxiliary\" section",
                      default=False)    
    parser.add_option("-A", "--auxiliary-on-file", dest="aux_filename",
                      help="write only \"Auxiliary\" section in file AUX_FILENAME, with one variable per line",
                      default="")
    parser.add_option("-b", "--buonds", dest="dumpBounds",
                      action="store_true", help="write \"Bounds\" section",
                      default=False)    
    parser.add_option("-B", "--bounds-on-file", dest="bounds_filename",
                      help="write only \"Bounds\" section in file BOUNDS_FILENAME, with one bound per line",
                      default="")
    parser.add_option("-V", "--version", dest="version",
                      help="version of the convexer to use: 2 or 3", 
                      default=3)
    parser.add_option("-O", "--optimize", dest="optimize",
                      action="store_true", help="perfom optimizations on OR translation (working only with -V 3; when -V 2 works, the optimization is automatically performed)",
                      default=False)    
    (options, args) = parser.parse_args()

# read and convex
def main():
    global options, auxiliary
    # possibly delete file for output
    if options.outputfilename and os.path.exists(options.outputfilename):
        os.remove(options.outputfilename)
    # catch input stream
    if options.inputfilename:
        input = open(options.inputfilename, 'r')
    else:
        input = sys.stdin
    out = Printer(options)
    # choose the version (2 or 3)
    if (int(options.version) == 2):
        YACC = yacc
        conv = convex2.Convex2(options, out)
    elif (int(options.version) == 3): # options.version == 3
        YACC = yacc3
        conv = convex3.Convex3(options, out)
    else:
        print "Error: version " + options.version + " of convexer not yet existing!"
        sys.exit()
    # - parse each input line (file or stdin) 
    # - convert it into a (set of) constraint
    # - print the result (file or stdout)
    # - possibly dump Bounds section, Auxiliary vars
    line = ""
    for li in input:
        # check if this is part of a bigger line, i.e. if it ends with a backslash
        if "\\" in li:
            line += li.replace("\\", " ")
            continue
        line += li.rstrip()
        # parse
        result = YACC.parser.parse(line)
        if result != [] and result != None:
            if options.verbose:
                out.output("INPUT: " + line.strip())
            res = conv.convex(result)
            for c in res:
                out.output(c)
        # in order for the final backslash mechanism to work, we need to reset 'line'
        line = ""
    if options.dumpBounds:
        out.output("Bounds")
        h = YACC.getBounds()
        inf = float('infinity')
        for var in h:
            if inf != abs(h[var][0]) and inf != abs(h[var][0]):
                out.output(str(h[var][0]) + " <= " + var + " <= " + str(h[var][1]))
            else:
                out.output(var + " free")
    if options.bounds_filename != "":
        bounds_file = open(options.bounds_filename, 'w')
        h = YACC.getBounds()
        inf = float('infinity')
        for var in h:
            if inf != abs(h[var][0]) and inf != abs(h[var][0]):
                bounds_file.write(str(h[var][0]) + " <= " + var + " <= " + str(h[var][1]) + "\n")
            else:
                bounds_file.write(var + " free" + "\n")
        bounds_file.close()
    if options.aux_filename != "":
        aux_file = open(options.aux_filename, 'w')
        for var in conv.auxiliary:
            aux_file.write(var + "\n")
        aux_file.close()
    if options.dumpAuxiliary:
        out.output("Auxiliary")
        aux = ""
        for var in conv.auxiliary:
            aux = aux + var + " "
        out.output(aux)
    if options.inputfilename:
        input.close()
    return



if (__name__=="__main__"):
    global options, args
    parse_command_line()
    if (int(options.version) != 2 and int(options.version) != 3):
        print "Error: version " + options.version + " of convexer not yet existing!"
        sys.exit()

    # Before importing modules, check for existence of already generated parser tables (from PLY) and possibly remove them
    if os.access("parsetab.py", os.F_OK):
        os.remove("parsetab.py")
    if os.access("parsetab.pyc", os.F_OK):
        os.remove("parsetab.pyc")
    if os.access("parser.out", os.F_OK):
        os.remove("parser.out")

    # Depending on the version, we use a different parser
    if (int(options.version) == 2):
        from convexer3.version2 import *
    else:
        from convexer3.version3 import *
    main()

