from __future__ import with_statement

import sys
import os
import pdb

from ..constr import constraint
from ..constr import guardedconstraint



class Printer:
    """
    The Printer class supplies `output' and `output_formula' functions, behaving according to given options.
    """

    def __init__(self, options):
        self.options = options

    def output(self, string):
        if self.options.outputfilename:
            with open(self.options.outputfilename, 'a+') as stream:
                stream.write(str(string) + "\n")
        else:
            print string
    
    def output_formula(self, ast):
        """
        Prints a formula, walking recursively in its ast
        """
        if ast[0] == 'AND':
            if self.options.outputfilename:
                with open(self.options.outputfilename, 'a+') as stream:
                    stream.write("{")
                    self.output_formula(ast[1])
                    stream.write("} & {")
                    self.output_formula(ast[2])
                    stream.write("}")
            else:
                print "{",
                self.output_formula(ast[1])
                print "} & {",
                self.output_formula(ast[2])
                print "}",
        elif ast[0] == 'OR':
            if self.options.outputfilename:
                with open(self.options.outputfilename, 'a+') as stream:
                    stream.write("{")
                    self.output_formula(ast[1])
                    stream.write("} | {")
                    self.output_formula(ast[2])
                    stream.write("}")
            else:
                print "{",
                self.output_formula(ast[1])
                print "} | {",
                self.output_formula(ast[2])
                print "}",
        elif ast[0] == 'NOT FORMULA' or ast[0] == 'NOT EQ':
            if self.options.outputfilename:
                with open(self.options.outputfilename, 'a+') as stream:
                    stream.write("!{")
                    self.output_formula(ast[1])
                    stream.write("}")
            else:
                print "!{",
                self.output_formula(ast[1])
                print "}",
        elif ast[0] == 'IMPLIES': # only implies in formulas
            if self.options.outputfilename:
                with open(self.options.outputfilename, 'a+') as stream:
                    stream.write(ast[1] + " -> ")
                    self.output_formula(ast[2])
            else:
                print ast[1] + " -> ",
                self.output_formula(ast[2])
        else:
            c = self.get_constraint(ast)
            if self.options.outputfilename:
                with open(self.options.outputfilename, 'a+') as stream:
                    stream.write(c)
            else:
                print c,


    """
    The following functions `get_constraint' and `get_rec' work on Abstract Syntax Trees
    """
    def get_constraint(self, ast):
        """
        Takes the AST of a constraint and returns a GuardedConstraint representing it
        """
        if self.options.verbose:
            self.output("get_constraint: " + str(ast))
        if ast[0] == 'IMPLIES': # we are in front of a guarded predicate ['IMPLIES', 'y', [:constraint:]]
            recres = self.get_constraint(ast[2])
            gc = guardedconstraint.GuardedConstraint()
            gc.setLhs(ast[1])
            gc.setRhs(recres)
            return gc
        recur = [self.get_rec(ast[1]), self.get_rec(ast[2])]
        res = recur[0].update(recur[1], True)
        # must change sign for b as it is considered as it were in the LHS
        res.setb(-res.getb())
        res.rel = ast[0]
        return res

    def get_rec(self, ast):
        """
        Recursive function for self.get_constraint()
        """
        if ast[0] == 'PLUS':
            return self.get_rec(ast[1]).update(self.get_rec(ast[2]), False)
        elif ast[0] == 'MINUS':
            return self.get_rec(ast[1]).update(self.get_rec(ast[2]), True)
        elif ast[0] == 'UMINUS':
            return self.get_rec(ast[1]).opposite()
        elif ast[0] == 'PARENS':
            return self.get_rec(ast[1])
        elif ast[0] == 'CONST':
            res = constraint.Constraint()
            res.setb(ast[1])
            return res
        elif ast[0] == 'MONOMIAL':
            res = constraint.Constraint()
            res.addTerm(ast[1].keys()[0], ast[1].values()[0]) # MONOMIAL has a dict with only one key
            return res
        else:
            print "Semantic Error (get_rec): " + str(ast)
            pdb.set_trace()
            sys.exit()
    

####### end of Printer class    
