#! /usr/bin/env python

#
#  Convexer 2.0
#
#      Lexical Analyzer Generator Spec
#
#      NEW: Supports multiple input line with backslash
#      NEW: Supports formulas among equations. Forbidden: the not equal operator.
#      NEW: Supports exponential format real numbers
#
#      authors: 
#           Federico Mari <mari@di.uniroma1.it>
#           Igor Melatti  <melatti@di.uniroma1.it>
#

import ply.lex as lex
import sys

# List of token names. To be imported in sw using the output lexer
tokens = (
   'REALNUMBER',
   'NUMBER',
   'PLUS',
   'MINUS',
   'TIMES',
   'DIVIDE',
   'LPAREN',
   'RPAREN',
   'LBRACE',
   'RBRACE',
   'IMPLIES',
   'ASSIGN',
   'TO',
   'INFTY',
   'LEQ',
   'GEQ',
   'EQ',
   'ID',
   'NOT',
   'AND',
   'OR'
)

# Regular expression rules for simple tokens
t_PLUS    = r'\+'
t_MINUS   = r'-'
t_TIMES   = r'\*'
t_DIVIDE  = r'/'
t_LPAREN  = r'\('
t_RPAREN  = r'\)'
t_LBRACE  = r'\{'
t_RBRACE  = r'\}'
t_IMPLIES = r'->'
t_ASSIGN  = r':='
t_TO      = r'\.\.'
t_LEQ     = r'<='
t_GEQ     = r'>='
t_EQ      = r'='
t_NOT     = r'!'
t_AND     = r'&'
t_OR      = r'\|'

# Regular expression rules with some action code
def t_REALNUMBER(t):
    r'\d+\.\d*((e|E)(\+|-){0,1}\d+){0,1}|\.\d+((e|E)(\+|-){0,1}\d+){0,1}|\d+(e|E)(\+|-){0,1}\d+'
    t.value = float(t.value)    
    return t

def t_NUMBER(t):
    r'\d+'
    t.value = int(t.value)    
    return t

# Track line numbers
def t_newline(t):
    r'\n+'
    t.lexer.lineno += len(t.value)

# infinity
def t_INFTY(t):
    r'infty'
    return t

# ID has an optional \' at the end (the apex character) for primed vars
def t_ID(t):
    r'[a-zA-Z_][a-zA-Z_0-9]*\'{0,1}'
    return t

# A string containing ignored characters
t_ignore  = ' \t'

# Error handling rule
def t_error(t):
    print "Illegal character '%s'" % t.value[0]
    t.lexer.skip(1)

# Comments have the C form: /* comment */
def t_COMMENT(t):
    r'/\*(.|\n)*?\*/'
    pass


# Build the lexer
lexer = lex.lex()


# just for debug
if (__name__ == "__main__"):
    for line in sys.stdin:
        lexer.input(line)
        while True:
            tok = lexer.token()
            if not tok: break
            print tok
